// Copyright (C) 2018  KINA Open
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published
// by the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

import 'dart:async';
import 'package:grpc/grpc.dart';
import 'moonshade.pb.dart';
import 'moonshade.pbgrpc.dart';

class Client {
  ClientChannel channel;
  DocumentClient documentStub;
  String address;
  int port;
  String username;
  String password;

  Client(this.address, this.port, this.username, this.password) {
    channel = new ClientChannel(address,
        port: port,
        options: const ChannelOptions(
            credentials: const ChannelCredentials.insecure()));
    documentStub = new DocumentClient(channel);
  }

  Future<Null> shutdown() async {
    await channel.shutdown();
  }

  Future<List> findOne(
      String storage, String db, String coll, String flt) async {
    var response = await documentStub.findOne(new FindOneRequest()
      ..token = (new Token()
        ..username = username
        ..password = password
        ..storage = storage
        ..database = db
        ..collection = coll)
      ..filter = flt);
    return [response.result, response.code];
  }

  Future<List> deleteOne(
      String storage, String db, String coll, String flt) async {
    var response = await documentStub.deleteOne(new DeleteOneRequest()
      ..token = (new Token()
        ..username = username
        ..password = password
        ..storage = storage
        ..database = db
        ..collection = coll)
      ..filter = flt);
    return [response.result, response.code];
  }

  Future<List> insertOne(
      String storage, String db, String coll, String doc) async {
    var response = await documentStub.insertOne(new InsertOneRequest()
      ..token = (new Token()
        ..username = username
        ..password = password
        ..storage = storage
        ..database = db
        ..collection = coll)
      ..document = doc);
    return [response.result, response.code];
  }

  Future<List> replaceOne(
      String storage, String db, String coll, String flt, String doc) async {
    var response = await documentStub.replaceOne(new ReplaceOneRequest()
      ..token = (new Token()
        ..username = username
        ..password = password
        ..storage = storage
        ..database = db
        ..collection = coll)
      ..filter = flt
      ..replace = doc);
    return [response.result, response.code];
  }

  Future<List> updateOne(
      String storage, String db, String coll, String flt, String doc) async {
    var response = await documentStub.updateOne(new UpdateOneRequest()
      ..token = (new Token()
        ..username = username
        ..password = password
        ..storage = storage
        ..database = db
        ..collection = coll)
      ..filter = flt
      ..update = doc);
    return [response.result, response.code];
  }
}
